##----------------------------------------------------------------------------##
##=Purpose: Exercise 5.1 of Beyond Classic Designs & Analysis for Health      ##
##          Data in Singapore (27 June to 8 July 2022)                        ##
##
##=Data required: 
##  - neonatal_jaundice.dta
##
##=R packages/libraries required:
##  - DoublingOfCases:
##    * logit_db: for estimating risk ratio from any standard epidemiological 
##                design by doubling the cases
##      link: https://github.com/nyilin/DoublingOfCases
##  - dplyr: 
##    * for processing data
##  - Epi:
##    * twoby2: For generating both RR and OR, esp RR as fisher.test can be 
##              used to get OR and its 95%CI.
##  - haven:
##    * read_dta: To read in recent versions of STATA dta files.
##
##=Created by:
##  - Francesca Graziano 
##  - Zilu Feng
##  - Zihan 
##  - Chuen Seng Tan
##  - Marie Reilly
##
##=Changes (if any):
##  -DDMMYYYY: Describe
##
##----------------------------------------------------------------------------##

##-Set the working directory to the data folder where the datasets are saved:
setwd("D:/nbox/Education/Marie_short_course/Singapore course 2022/20220602/exercises/data")

##-R libraries to install:
#Install Package `devtools` required before installing DoublingOfCases:
#install.packages("devtools")
#devtools::install_github("nyilin/DoublingOfCases")
#install.packages(c("dplyr","Epi","haven"))
##-Load the R libraries
library(DoublingOfCases)
library(dplyr)
library(Epi)
library(haven)


################################################################################
##                               Question 1      
##
##1- In a population-based study of  risk factors for hemolytic and 
##   non-haemolytic neonatal jaundice (Lee et al, Acta Paediatrika, 2016, 
##   DOI: 10.1111/apa.13470), the crude RR of haemolytic jaundice from 1992-2002 
##   for singleton infants born preterm was 16.5, and the crude OR was 27.8. 
##
##   The dataset neonatal_jaundice.dta contains a summary of these data, where 
##   the variable num_neonates provides the numbers of infants in each of the 
##   categories defined by the outcome of interest event (diagnosis of jaundice: 
##   event), the main exposure of interest (preterm delivery: preterm) and a few 
##   other variables (mother age 35 or older: age35, BMI category: overobes  
##   [0=neither, 1= overweight, 2 =obese], multiparity: multipar, 
##   sex of neonate: female).
################################################################################

################################################################################
##                               Question 1 (i)     
##
##-(i) Expand the data to one record per infant, tabulate exposure by outcome, 
##     and find the crude RR and crude OR for the association between preterm 
##     delivery and jaundice
################################################################################

#1- read in the data
dat <- read_dta("neonatal_jaundice.dta")

#2- declare overobes as a factor:
dat$overobes <- factor(dat$overobes,
                       levels = c(0,1,2),
                       labels = c("neither", "overweight", "obese"))

#3- expand the data
dat.expand<-dat[rep(1:nrow(dat),times=dat$num_neonates),]

#4- generate the RR and OR (use 1-variable to ensure the exposed and event is at the top left corner of the 2x2 table
Epi::twoby2(1-dat.expand$preterm,1-dat.expand$event)

#crude RR=16.5 and crude OR=27.8

################################################################################
##                               Question 1 (ii)     
##
##-(ii) Run a logistic regression to obtain the OR for preterm, adjusted for 
##      maternal age, BMI, parity and sex of infant. Verify that your adjusted 
##      OR is close to the value reported in the last column of the table at 
##      the end of this document (analysis of a case-control sample from the 
##      full data that is adjusted for maternal smoking).
################################################################################

#1- fit logistic regression
logit.obj<-glm(event~preterm+age35+overobes+multipar+female,
               data=dat.expand,family=binomial)
#2- key results
round(cbind(exp(cbind(OR=logit.obj$coefficients,
                      confint.default(logit.obj))),
            p.value=summary(logit.obj)$coefficients[,"Pr(>|z|)"]),3)

#Verified that the value generated is close to the value reported

################################################################################
##                               Question 1 (iii)     
##
##-(iii) Modify the dataset using the double-the-cases method. Tabulate 
##       exposure by outcome in the modified data, compare to the 
##       exposure-by-outcome table from (i) to verify that the frequencies are 
##       correct, i.e.
##
##       [see exercise]
################################################################################

#1- identify only the cases
cases<-dat[which(dat$event==1),]

#2- expand the cases
cases.expand<-cases[rep(1:nrow(cases),times=cases$num_neonates),]

#3- change the case status to control status
cases.expand$event<-0

#4- generate the modified data
dat.mod<-rbind(dat.expand, 
               cases.expand)

#5- anticipated 2x2 table for modified data
table(dat.expand$preterm,dat.expand$event)
527376+13881
11093+8116

#6- 2x2 table for modified data
table(dat.mod$preterm,dat.mod$event)


################################################################################
##                               Question 1 (iv)     
##
##-(iv) Run a logistic regression analysis of the modified data to obtain the 
##      adjusted RR for preterm and compare to the value reported in the first 
##      column of the table below from the full data.
################################################################################

#1- fit logistic regression
logit.obj.mod<-glm(event~preterm+age35+overobes+multipar+female,
                   data=dat.mod,family=binomial("logit"))
exp(logit.obj.mod$coefficients)

#adjusted RR= 15.42, in close agreement with RR in full cohort.


################################################################################
##                               Question 1 (v)     
##
##-(v) Select a 1:2 case-control sample from the cohort using simple random 
##     sampling of controls, and compare the estimates of the adjusted OR from 
##     logistic regression and adjusted RR from weighted logistic regression to 
##     the values obtained from the full cohort in (ii) and (iv) respectively.
################################################################################

#1- number of controls to sample
no.controls<-2*sum(dat.expand$event==1)

#2- sample the controls
set.seed(1234)
sampled.controls <- dat.expand %>%
  dplyr::filter(event == 0)  %>% 
  dplyr::slice_sample(n=no.controls, replace = FALSE)
dim(sampled.controls)

#3 - extract all cases
all.cases <- dat.expand %>%
  dplyr::filter(event == 1) 
dim(all.cases)

#4 -combine the sampled controls and all cases
dat.cc <- rbind(sampled.controls,
                all.cases)

#5- estimates of the adjusted OR from logistic regression 
logit.obj.cc<-glm(event~preterm+age35+overobes+multipar+female,
                  data=dat.cc,family=binomial("logit"))
exp(logit.obj.cc$coefficients)

#adjusted OR=26.80, in close agreement with it in logistic regression in full cohort in (ii).

#6- estimates of the adjusted RR from weighted logistic regression 
#6a- generate weights
tot.controls <- sum(dat.expand$event==0)
dat.cc$weights <- ifelse(dat.cc$event==1,
                         1, tot.controls/no.controls)

#6b- identify only the cases
cases.cc<-dat.cc %>%
  dplyr::filter(event==1)

#6c- change the case status to control status
cases.cc$event<-0

#7- generate the modified data
dat.cc.mod<-rbind(dat.cc, 
                  cases.cc)

#8- adjusted RR using weighted logistic regression from doubling the cases 
wt.logit.obj.cc<-glm(event~preterm+age35+overobes+multipar+female,
                      data=dat.cc.mod,family=binomial("logit"),
                      weights = weights)
exp(wt.logit.obj.cc$coefficients)

#adjusted RR=15.16, in close agreement with adjusted RR in logistic regression in full cohort in (iv). The variance is invalid.


################################################################################
##                               Question 1 (vi)     
##
##- (vi) Repeat the selection of the 1:2 case-control sample from part (v), 
##       but this time select controls that are frequency matched on maternal 
##       age and infant sex. Assign appropriate weights to cases and controls. 
##       Double the cases and verify that an unweighted logistic regression 
##       provides a biased estimate of the adjusted RR, but a weighted logistic 
##       regression (weighted by the inverse of the sampling fractions) 
##       provides an unbiased estimate.
################################################################################

# function to sample 2 controls for each case for the data (dd) and return the case-control data
srs<-function(dd){
  
  no.controls<-2*sum(dd$event==1)
  
  #2- sample the controls
  s.con <- dd %>%
    dplyr::filter(event == 0)  %>% 
    dplyr::slice_sample(n=no.controls, replace = FALSE)

  #3 - extract all cases
  a.cas <- dd %>%
    dplyr::filter(event == 1) 
  
  #4 -combine the sampled controls and all cases
  dd.cc <- rbind(s.con,
                 a.cas)
  return(dd.cc)
}

#1- sample 2 controls for each case frequency matched on maternal age and infant sex
set.seed(1234)
dat.mcc <- dat.expand %>%
  dplyr::group_by(interaction(age35,female)) %>% 
  dplyr::group_map(~srs(.x)) 

#2- make the sampled case-control data from a list to a tabular format 
dat.mcc<-do.call(rbind,dat.mcc)

#3- get the frequencies from the sample and population by matching variables to generate the weights
sam.n<-table(interaction(dat.mcc$age35,dat.mcc$female),
             dat.mcc$event)
pop.n<-table(interaction(dat.expand$age35,dat.expand$female),
             dat.expand$event)
sam.n
pop.n

#checking if the sampled controls is doubled the cases
all(sam.n[,2]==pop.n[,2]) & all(sam.n[,1]==(2*pop.n[,2]))

#4- generate the weights by setting it as 1 first and assign a weight to the controls
dat.mcc <- dat.mcc %>%
  dplyr::mutate(weights=1) %>% 
  dplyr::mutate(weights=replace(weights,age35==0 & female==0 & event==0,pop.n[1,1]/sam.n[1,1])) %>%
  dplyr::mutate(weights=replace(weights,age35==1 & female==0 & event==0,pop.n[2,1]/sam.n[2,1])) %>%
  dplyr::mutate(weights=replace(weights,age35==0 & female==1 & event==0,pop.n[3,1]/sam.n[3,1])) %>%
  dplyr::mutate(weights=replace(weights,age35==1 & female==1 & event==0,pop.n[4,1]/sam.n[4,1]))

#checking if sampling weights are right
unique(dat.mcc[,c("age35","female","weights")])
pop.n[,1]/sam.n[,1]

#5- doubling the cases
#5a- identify only the cases
cases.mcc<-dat.mcc[which(dat.mcc$event==1),]

#5b- change the case status to control status
cases.mcc$event<-0

#5c- genreate the modified data
dat.mcc.mod<-rbind(dat.mcc, 
                   cases.mcc)

table(dat.mcc.mod$event)
table(dat.mcc$event)
43994+21997

#6- adjusted RR from unweighted logistic regression from doubling of cases  
logit.obj.mcc<-glm(event~preterm+age35+overobes+multipar+female,
                  data=dat.mcc.mod,family=binomial("logit"))
#key results
exp(logit.obj.mcc$coefficients)
#adjusted RR=3.56, which is biased     

#7- adjusted RR from weighted logistic regression from doubling of cases 
wt.logit.obj.mcc<-glm(event~preterm+age35+overobes+multipar+female,
                      data=dat.mcc.mod,family=binomial("logit"),
                      weights=weights)
#key results
exp(wt.logit.obj.mcc$coefficients)
#adjusted RR=15.40, in close agreement with adjusted RR in logistic regression in full cohort in (iv).


################################################################################
##                               Question 1 (vii)     
##
##-(vii) To obtain standard errors for the estimates above, further calculations
##       are required. These are automatically provided if the data are 
##       analyzed in R using the "DoublingOfCases" package available from 
##       https://github.com/nyilin/DoublingOfCases. If you are using R, 
##       reanalyze the data now using this package with the logit_db function. 
##       For the Stata output, comment the invalid variance is too small or 
##       too large.
################################################################################

#1- Adjusted RR for weighted logistic regression with 1:2 unmatched case-control 
db.cc <- logit_db(event~preterm+age35+overobes+multipar+female, 
                  data = as.data.frame(dat.cc), weight_name = "weights")
summary(db.cc)$coefficients
confint(db.cc, exp = TRUE)

#2- Adjusted RR for weighted logistic regression with 1:2 matched case-control 
db.mcc <- logit_db(event~preterm+age35+factor(overobes)+multipar+female, 
                  data = as.data.frame(dat.mcc), weight_name = "weights")
summary(db.mcc)$coefficients
confint(db.mcc, exp = TRUE)
