//----------------------------------------------------------------------------//
//=Purpose: Exercise 5.1 of Beyond Classic Designs & Analysis for Health      //
//          Data in Singapore (27 June to 8 July 2022)                        //
//
//=Data required: 
//  - neonatal_jaundice.dta
//
//=Stata packages/libraries required:
//
//=Created by:
//  - Zihan 
//  - Zilu Feng
//  - Francesca Graziano 
//  - Chuen Seng Tan
//  - Marie Reilly
//
//=Changes (if any):
//  -DDMMYYYY: Describe
//
//----------------------------------------------------------------------------//

//-Set the working directory to the data folder where the datasets are saved:
cd "D:/nbox/Education/Marie_short_course/Singapore course 2022/20220602/exercises/data"

////////////////////////////////////////////////////////////////////////////////
//                               Question 1      
//
//1- In a population-based study of  risk factors for hemolytic and 
//   non-haemolytic neonatal jaundice (Lee et al, Acta Paediatrika, 2016, 
//   DOI: 10.1111/apa.13470), the crude RR of haemolytic jaundice from 1992-2002 
//   for singleton infants born preterm was 16.5, and the crude OR was 27.8. 
//
//   The dataset neonatal_jaundice.dta contains a summary of these data, where 
//   the variable num_neonates provides the numbers of infants in each of the 
//   categories defined by the outcome of interest event (diagnosis of jaundice: 
//   event), the main exposure of interest (preterm delivery: preterm) and a few 
//   other variables (mother age 35 or older: age35, BMI category: overobes  
//   [0=neither, 1= overweight, 2 =obese], multiparity: multipar, 
//   sex of neonate: female).
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (i)     
//
//-(i) Expand the data to one record per infant, tabulate exposure by outcome, 
//     and find the crude RR and crude OR for the association between preterm 
//     delivery and jaundice
////////////////////////////////////////////////////////////////////////////////

//load the data
use neonatal_jaundice.dta, clear

//expanded to have one record for each individual as follows:
expand num_neonates
drop num_neonates

//calculate crude RR:
tab event preterm
csi 8116 13881 11093 527376

//crude RR=16.5

//calculate crude OR:
logistic event preterm

//crude OR=27.8
	
	
////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (ii)     
//
//-(ii) Run a logistic regression to obtain the OR for preterm, adjusted for 
//      maternal age, BMI, parity and sex of infant. Verify that your adjusted 
//      OR is close to the value reported in the last column of the table at 
//      the end of this document (analysis of a case-control sample from the 
//      full data that is adjusted for maternal smoking).
////////////////////////////////////////////////////////////////////////////////

logistic event preterm age35 i.overobes multipar female

//adjusted OR=26.5, in agreement with example 5.3.


////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (iii)     
//
//-(iii) Modify the dataset using the double-the-cases method. Tabulate 
//       exposure by outcome in the modified data, compare to the 
//       exposure-by-outcome table from (i) to verify that the frequencies are 
//       correct, i.e.
//
//       [see exercise]
////////////////////////////////////////////////////////////////////////////////

//2*2 table in original dataset:
tab preterm event

//use "double-the-cases" method:
preserve 
keep if event==1
replace event=0
save 1, replace
restore

append using 1

//2*2 table in "double-the-cases" dataset:
tab preterm event			


////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (iv)     
//
//-(iv) Run a logistic regression analysis of the modified data to obtain the 
//      adjusted RR for preterm and compare to the value reported in the first 
//      column of the table below from the full data.
////////////////////////////////////////////////////////////////////////////////

//logistic model in "double-the-cases" dataset:
logistic event preterm age35 i.overobes multipar female

//adjusted RR= 15.42, in close agreement with RR in full cohort.

////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (v)     
//
//-(v) Select a 1:2 case-control sample from the cohort using simple random 
//     sampling of controls, and compare the estimates of the adjusted OR from 
//     logistic regression and adjusted RR from weighted logistic regression to 
//     the values obtained from the full cohort in (ii) and (iv) respectively.
////////////////////////////////////////////////////////////////////////////////

//load the data and expand:
use neonatal_jaundice.dta, clear
expand num_neonates
drop num_neonates

tab event 

//we need to select 21997 case and 21997*2=43994 non-cases
//set a seed using a random number:
set seed 1810

//generate shuffle:
gen u = runiform()
sort event u

//generate a variable (select=0/1) to specify ppl selected:
gen select = 0

//selects all (21997) event==1 and 43994 event==0:
by event: replace select = 1 if _n <= 43994

//check the results of selection: 
tabulate select event

//only keep ppl selected:
drop if select==0

//Weighted logistic regression on second-stage sample:
gen weight = 21997/21997 if event == 1
replace weight = 538469/43994 if event == 0

//logistic model (get adjusted OR):
logistic event preterm age35 i.overobes multipar female

//adjusted OR= 26.91, in close agreement with it in logistic regression in full cohort in (ii).

**use "double-the-cases" method to calculate adjusted RR**

preserve 
keep if event==1
replace event=0
save 2, replace
restore

append using 2

//logistic model in "double-the-cases" dataset (get adjusted RR):
logistic event preterm age35 i.overobes multipar female [pw = weight]
	
//adjusted RR= 15.21, in close agreement with adjusted RR in logistic regresion in full cohort in (iv). The variance is invalid.


////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (vi)     
//
//- (vi) Repeat the selection of the 1:2 case-control sample from part (v), 
//       but this time select controls that are frequency matched on maternal 
//       age and infant sex. Assign appropriate weights to cases and controls. 
//       Double the cases and verify that an unweighted logistic regression 
//       provides a biased estimate of the adjusted RR, but a weighted logistic 
//       regression (weighted by the inverse of the sampling fractions) 
//       provides an unbiased estimate.
////////////////////////////////////////////////////////////////////////////////

//load the data
use neonatal_jaundice.dta, clear
expand num_neonates
drop num_neonates
gen id=_n
save expand, replace

//coding event age35 female
//from number of case in each stratum, we need to select 21564, 15872, 3716, 2842 controls in each correponding stratum. 
gen control_strata=1 if event==0 & age35==0 & female==0
replace control_strata=2 if event==0 & age35==0 & female==1
replace control_strata=3 if event==0 & age35==1 & female==0
replace control_strata=4 if event==0 & age35==1 & female==1

//set a seed using a random number:
set seed 1810

//generate shuffle:
gen u = runiform()
sort control_strata u

//generate a variable (select=0/1) to specify ppl selected：
gen select = 0

//prepare case-control dataset:
preserve
replace select=1 if event==1
keep if event==1
save event, replace
restore

//randomly select twice as many controls:
preserve
keep if control_strata==1
replace select = 1 if _n <= 21564 & control_strata==1
save c1, replace
restore

preserve
keep if control_strata==2
replace select = 1 if _n <= 15872 & control_strata==2
save c2, replace
restore

preserve
keep if control_strata==3
replace select = 1 if _n <= 3716 & control_strata==3
save c3, replace
restore

keep if control_strata==4
replace select = 1 if _n <= 2842 & control_strata==4

append using c3
append using c2
append using c1
append using event 
drop if select==0

codebook event

//calculate weights:
gen weight=1 if event==1
replace weight=236651/21564 if event==0 & age35==0 & female==0
replace weight=226563/15872 if event==0 & age35==0 & female==1
replace weight=38361/3716 if event==0 & age35==1 & female==0
replace weight=36894/2842 if event==0 & age35==1 & female==1

//use "double-the-cases" method to calculate adjusted RR:

preserve 
keep if event==1
replace event=0
save doublecases, replace
restore

append using doublecases

//unweighted logistic regression model in "double-the-cases" dataset:
logistic event preterm age35 i.overobes multipar female

//adjusted RR=3.55, which is biased
	
//weighted logistic model in "double-the-cases" dataset:
logistic event preterm age35 i.overobes multipar female [pw = weight]

//adjusted RR= 15.08, in close agreement with adjusted RR in logistic regression in full cohort in (iv).	


////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (vii)     
//
//-(vii) To obtain standard errors for the estimates above, further calculations
//       are required. These are automatically provided if the data are 
//       analyzed in R using the "DoublingOfCases" package available from 
//       https://github.com/nyilin/DoublingOfCases. If you are using R, 
//       reanalyze the data now using this package with the logit_db function. 
//       For the Stata output, comment the invalid variance is too small or 
//       too large.
////////////////////////////////////////////////////////////////////////////////

//0.25 (SE from the full cohort) vs.0.38 (SE from the case-control data), so the invalid variance is large.

